@@ -7,10 +7,11 @@ class Propshaft::Server
   end
 
   def call(env)
+    execute_cache_sweeper_if_updated
     path, digest = extract_path_and_digest(env)
 
     if (asset = @assembly.load_path.find(path)) && asset.fresh?(digest)
-      compiled_content = @assembly.compilers.compile(asset)
+      compiled_content = asset.compiled_content
 
       [
         200,
@@ -18,7 +19,7 @@ class Propshaft::Server
           Rack::CONTENT_LENGTH  => compiled_content.length.to_s,
           Rack::CONTENT_TYPE    => asset.content_type.to_s,
           VARY                  => "Accept-Encoding",
-          Rack::ETAG            => asset.digest,
+          Rack::ETAG            => "\"#{asset.digest}\"",
           Rack::CACHE_CONTROL   => "public, max-age=31536000, immutable"
         },
         [ compiled_content ]
@@ -35,10 +36,8 @@ class Propshaft::Server
   private
     def extract_path_and_digest(env)
       full_path = Rack::Utils.unescape(env["PATH_INFO"].to_s.sub(/^\//, ""))
-      digest    = full_path[/-([0-9a-zA-Z]{7,128})\.(?!digested)([^.]|.map)+\z/, 1]
-      path      = digest ? full_path.sub("-#{digest}", "") : full_path
 
-      [ path, digest ]
+      Propshaft::Asset.extract_path_and_digest(full_path)
     end
 
     if Gem::Version.new(Rack::RELEASE) < Gem::Version.new("3")
@@ -46,4 +45,8 @@ class Propshaft::Server
     else
       VARY = "vary"
     end
+
+    def execute_cache_sweeper_if_updated
+      @assembly.load_path.cache_sweeper.execute_if_updated
+    end
 end
